#
# libjlc.s - The James Ladd Code Library. 
#            A set of helpful API's.
# 
# Copyright (c) 2006 James Ladd.
# http://www.hyperserve.org
#
# Please read the LICENSE file.
#
# Documentation on these API's can be found in
# the 'C' header file libjlc.h
#

.intel_syntax noprefix

.include "system.inc"
.include "libjlc.inc"

.section .text

# --------------------------------------
.ifndef WIN
.type _jlc_lib_attach, @function
.endif
.global _jlc_lib_attach
_jlc_lib_attach:
        push ebp
        mov ebp, esp

        # Params are:
        # jlclib* [ebp +8]

        # No local variables (yet).

        # Right now there is no initialization needed during an attach
        # so we return a placeholder value in [ebp +8]

        mov edx, [ebp +8]
        mov [edx], dword ptr 3
        mov eax, 0

        mov esp, ebp
        pop ebp
        ret

# --------------------------------------
.ifndef WIN
.type _jlc_lib_detach, @function
.endif
.global _jlc_lib_detach
_jlc_lib_detach:
        push ebp
        mov ebp, esp

        xor eax, eax

        mov esp, ebp
        pop ebp
        ret


# --------------------------------------
.ifndef WIN
.type _jlc_tpool_open, @function
.endif
.global _jlc_tpool_open
_jlc_tpool_open:
        push ebp
        mov ebp, esp

        # Params are:
        # jlclib [ebp +8], tpool* [ebp +12], tcount [ebp +16], qcount [ebp +20]
        # TODO: Make use of qcount as we ignore it right now.

        # No local variables (yet).

        # Steps:
        # 1. Allocate the thread pool structure and,
        # 2. Initialize the thread pool structure and,
        # 3. Create the required threads waiting on a 'run' barrier and,
        # 4. Return the pointer to the thread pool [ebp +12]. 

        # 1.
        # for now I use a static piece of memory until we get the API
        # working. TODO: Make a call to a allocator for this.
        mov edx, offset temp_tpool

        # 4.
        mov ecx, [ebp +12]             # store in tpool* for result/later
        mov [ecx], edx

        mov ecx, [ebp +8]              # store jlclib handle into structure.
        mov [edx +TPOOL.jlclib], ecx

        # 2. 

        # 3. 

        xor eax, eax

        mov esp, ebp
        pop ebp
        ret

# --------------------------------------
.ifndef WIN
.type _jlc_tpool_close, @function
.endif
.global _jlc_tpool_close
_jlc_tpool_close:
        push ebp
        mov ebp, esp

        xor eax, eax

        mov esp, ebp
        pop ebp
        ret


# --------------------------------------
.ifndef WIN
.type _jlc_tpool_run, @function
.endif
.global _jlc_tpool_run
_jlc_tpool_run:
        push ebp
        mov ebp, esp

        xor eax, eax

        mov esp, ebp
        pop ebp
        ret

# --------------------------------------
.ifndef WIN
.type _jlc_tpool_submit, @function
.endif
.global _jlc_tpool_submit
_jlc_tpool_submit:
        push ebp
        mov ebp, esp

        xor eax, eax

        mov esp, ebp
        pop ebp
        ret

